package org.kohsuke.rng2srng;

import java.io.IOException;

import javax.xml.transform.Result;
import javax.xml.transform.dom.DOMResult;

import org.kohsuke.rng2srng.datatype.DatatypeLibraryFactoryImpl;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.thaiopensource.relaxng.IncorrectSchemaException;
import com.thaiopensource.relaxng.impl.Pattern;
import com.thaiopensource.relaxng.impl.SchemaBuilderImpl;
import com.thaiopensource.relaxng.impl.SchemaPatternBuilder;
import com.thaiopensource.relaxng.parse.Parseable;
import com.thaiopensource.relaxng.parse.compact.CompactParseable;
import com.thaiopensource.relaxng.parse.sax.SAXParseable;
import com.thaiopensource.relaxng.util.Jaxp11XMLReaderCreator;

/**
 * Convenience methods that makes it easy to use
 * this library from other applications.
 * 
 * @author Kohsuke Kawaguchi (kk@kohsuke.org)
 */
public class Translator {
    
    /**
     * Translates RELAX NG XML syntax into a DOM tree that represents
     * simplified RELAX NG syntax.
     * 
     * @param errorHandler
     *      Receives any parsing error.
     */
    public static Node translateXMLToDOM( InputSource source, ErrorHandler errorHandler )
        throws IOException, SAXException, IncorrectSchemaException {
        
        DOMResult result = new DOMResult();
        translateXmlToResult(source,errorHandler,result);
        return result.getNode();
    }

    /**
     * Translates RELAX NG compact syntax into a DOM tree that represents
     * simplified RELAX NG syntax.
     * 
     * @param errorHandler
     *      Receives any parsing error.
     */
    public static Node translateCompactToDOM( InputSource source, ErrorHandler errorHandler )
        throws IOException, SAXException, IncorrectSchemaException {
        
        DOMResult result = new DOMResult();
        translateCompactToResult(source,errorHandler,result);
        return result.getNode();
    }


    /**
     * Translates XML syntax and send the output to the specified
     * Result object.
     */
    public static void translateXmlToResult( InputSource source, ErrorHandler errorHandler, Result result )
        throws IOException, SAXException, IncorrectSchemaException {
        
        Writer.write( loadXml(source,errorHandler), result );
    }

    /**
     * Translates Compact syntax and send the output to the specified
     * Result object.
     */
    public static void translateCompactToResult( InputSource source, ErrorHandler errorHandler, Result result )
        throws IOException, SAXException, IncorrectSchemaException {
        
        Writer.write( loadCompact(source,errorHandler), result );
    }



    // TODO: more 


    private static Pattern loadXml( InputSource is, ErrorHandler errorHandler )
        throws SAXException, IOException, IncorrectSchemaException {
        
        return loadGrammar( new SAXParseable(new Jaxp11XMLReaderCreator(),is,errorHandler), errorHandler );
    }
    
    private static Pattern loadCompact( InputSource is, ErrorHandler errorHandler )
        throws SAXException, IOException, IncorrectSchemaException {
        
        return loadGrammar( new CompactParseable(is,errorHandler), errorHandler );
    }
    
    private static Pattern loadGrammar( Parseable source, ErrorHandler errorHandler )
        throws SAXException, IOException, IncorrectSchemaException {
        
        return SchemaBuilderImpl.parse(
            source,
            errorHandler,
            new DatatypeLibraryFactoryImpl(),
            new SchemaPatternBuilder() );
    }
}
